'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var prettier = require('prettier');
var svelte = require('svelte');

function isASTNode(n) {
    return 'html' in n && 'css' in n && 'js' in n;
}

const { concat, join, line, group, indent, softline, hardline } = prettier.doc.builders;
function print(path, options, print) {
    const n = path.getValue();
    if (!n) {
        return '';
    }
    if (isASTNode(n)) {
        const parts = [path.call(print, 'html')];
        if (n.css) {
            n.css.type = 'Style';
            n.css.content.type = 'StyleProgram';
            parts.push(path.call(print, 'css'));
        }
        if (n.js) {
            n.js.type = 'Script';
            parts.push(path.call(print, 'js'));
        }
        return group(join(hardline, parts));
    }
    const node = n;
    switch (node.type) {
        case 'Fragment':
            const children = node.children;
            return concat([
                printChildren(path, print, {
                    skipFirst: true,
                    filter: (node, i) => {
                        if (i === 0 && node.type === 'Text' && node.data.trim() === '') {
                            return false;
                        }
                        let include = false;
                        for (let j = i; j < children.length; j++) {
                            const child = children[j];
                            if (!(child.type === 'Text' && child.data.trim() === '')) {
                                include = true;
                                break;
                            }
                        }
                        return include;
                    },
                }),
                hardline,
            ]);
        case 'Text':
            return join(hardline, node.data
                .replace(/\n(?!\n)/g, '')
                .replace(/[ \t]+/g, ' ')
                .split(/\n/g));
        case 'Element':
        case 'Component':
        case 'Slot':
        case 'Window':
        case 'Head':
        case 'Title':
            return group(concat([
                '<',
                node.name,
                indent(group(concat([
                    node.type === 'Component' && node.expression
                        ? concat([
                            line,
                            'this={',
                            printJS(path, print, 'expression'),
                            '}',
                        ])
                        : '',
                    ...path.map(childPath => childPath.call(print), 'attributes'),
                ]))),
                node.children.length ? '>' : ' />',
                indent(printChildren(path, print)),
                node.children.length ? concat([softline, '</', node.name, '>']) : '',
            ]));
        case 'Script':
        case 'Style':
            return group(concat([
                '<',
                node.type.toLowerCase(),
                indent(group(concat(path.map(childPath => childPath.call(print), 'attributes')))),
                '>',
                indent(path.call(print, 'content')),
                hardline,
                '</',
                node.type.toLowerCase(),
                '>',
                hardline,
            ]));
        case 'Identifier':
            return node.name;
        case 'Attribute': {
            const hasLoneMustacheTag = node.value !== true &&
                node.value.length === 1 &&
                node.value[0].type === 'MustacheTag';
            let isAttributeShorthand = node.value !== true &&
                node.value.length === 1 &&
                node.value[0].type === 'AttributeShorthand';
            // Convert a={a} into {a}
            if (hasLoneMustacheTag) {
                const expression = node.value[0].expression;
                isAttributeShorthand =
                    expression.type === 'Identifier' && expression.name === node.name;
            }
            if (isAttributeShorthand) {
                return concat([line, '{', node.name, '}']);
            }
            const def = [line, node.name];
            if (node.value !== true) {
                def.push('=');
                const quotes = !hasLoneMustacheTag;
                quotes && def.push('"');
                def.push(...path.map(childPath => childPath.call(print), 'value'));
                quotes && def.push('"');
            }
            return concat(def);
        }
        case 'MustacheTag':
            return concat(['{', printJS(path, print, 'expression'), '}']);
        case 'IfBlock': {
            const def = [
                '{#if ',
                printJS(path, print, 'expression'),
                '}',
                indent(printChildren(path, print)),
                line,
            ];
            if (node.else) {
                def.push(path.call(print, 'else'));
            }
            def.push('{/if}');
            return group(concat(def));
        }
        case 'ElseBlock': {
            // Else if
            if (node.children.length === 1 && node.children[0].type === 'IfBlock') {
                const ifNode = node.children[0];
                const def = [
                    '{:elseif ',
                    path.map(ifPath => printJS(path, print, 'expression'), 'children')[0],
                    '}',
                    indent(path.map(ifPath => printChildren(ifPath, print), 'children')[0]),
                    line,
                ];
                if (ifNode.else) {
                    def.push(path.map(ifPath => ifPath.call(print, 'else'), 'children')[0]);
                }
                return group(concat(def));
            }
            return group(concat(['{:else}', indent(printChildren(path, print)), line]));
        }
        case 'EachBlock': {
            const def = [
                '{#each ',
                printJS(path, print, 'expression'),
                ' as ',
                printJS(path, print, 'context'),
            ];
            if (node.index) {
                def.push(', ', node.index);
            }
            if (node.key) {
                def.push(' (', printJS(path, print, 'key'), ')');
            }
            def.push('}', indent(printChildren(path, print)), line);
            if (node.else) {
                def.push(path.call(print, 'else'));
            }
            def.push('{/each}');
            return group(concat(def));
        }
        case 'AwaitBlock': {
            return group(concat([
                group(concat(['{#await ', printJS(path, print, 'expression'), '}'])),
                indent(path.call(print, 'pending')),
                line,
                group(concat(['{:then', node.value ? ' ' + node.value : '', '}'])),
                indent(path.call(print, 'then')),
                line,
                group(concat(['{:catch', node.error ? ' ' + node.error : '', '}'])),
                indent(path.call(print, 'catch')),
                line,
                '{/await}',
            ]));
        }
        case 'ThenBlock':
        case 'PendingBlock':
        case 'CatchBlock':
            return printChildren(path, print);
        case 'EventHandler':
            return concat([
                line,
                'on:',
                node.name,
                node.expression ? concat(['=', '"', printJS(path, print, 'expression'), '"']) : '',
            ]);
        case 'Binding':
            return concat([
                line,
                'bind:',
                node.name,
                node.value.type === 'Identifier' && node.value.name === node.name
                    ? ''
                    : concat(['=', '"', printJS(path, print, 'value'), '"']),
            ]);
        case 'DebugTag':
            return concat([
                '{@debug',
                node.identifiers.length > 0
                    ? concat([' ', join(', ', path.map(print, 'identifiers'))])
                    : '',
                '}',
            ]);
        case 'Ref':
            return concat([line, 'ref:', node.name]);
        case 'Comment':
            return group(concat(['<!--', node.data, '-->']));
        case 'Transition':
            const kind = node.intro && node.outro ? 'transition' : node.intro ? 'in' : 'out';
            return concat([
                line,
                kind,
                ':',
                node.name,
                node.expression ? concat(['=', '"', printJS(path, print, 'expression'), '"']) : '',
            ]);
        case 'Action':
            return concat([
                line,
                'use:',
                node.name,
                node.expression ? concat(['=', '"', printJS(path, print, 'expression'), '"']) : '',
            ]);
        case 'Animation':
            return concat([
                line,
                'animate:',
                node.name,
                node.expression ? concat(['=', '"', printJS(path, print, 'expression'), '"']) : '',
            ]);
        case 'RawMustacheTag':
            return concat(['{@html ', printJS(path, print, 'expression'), '}']);
        case 'Spread':
            return concat([line, '{...', printJS(path, print, 'expression'), '}']);
    }
    throw new Error('unknown node type: ' + node.type);
}
function printChildren(path, print, { skipFirst = false, filter = (node, i) => true } = {}) {
    const children = [];
    let i = 0;
    let isFirst = true;
    path.each(childPath => {
        const child = childPath.getValue();
        const index = i;
        i++;
        if (!filter(child, index)) {
            return;
        }
        if (!(isFirst && skipFirst) && child.type !== 'Text' && child.type !== 'MustacheTag') {
            children.push(hardline);
        }
        children.push(childPath.call(print));
        isFirst = false;
    }, 'children');
    return concat(children);
}
function printJS(path, print, name) {
    if (!name) {
        path.getValue().isJS = true;
        return path.call(print);
    }
    path.getValue()[name].isJS = true;
    return path.call(print, name);
}

const { builders: { concat: concat$1, hardline: hardline$1 }, utils: { removeLines }, } = prettier.doc;
function embed(path, print, textToDoc, options) {
    const node = path.getNode();
    if (node.isJS) {
        return removeLines(textToDoc(getText(node, options), {
            parser: expressionParser,
            singleQuote: true,
        }));
    }
    switch (node.type) {
        case 'Program':
            const script = getText(node, options);
            return concat$1([hardline$1, nukeLastLine(textToDoc(script, { parser: 'babylon' }))]);
        case 'StyleProgram':
            return concat$1([hardline$1, nukeLastLine(textToDoc(node.styles, { parser: 'css' }))]);
    }
    return null;
}
function getText(node, options) {
    return options.originalText.slice(options.locStart(node), options.locEnd(node));
}
function expressionParser(text, parsers) {
    const ast = parsers.babylon(`(${text})`);
    return {
        type: 'File',
        program: ast.program.body[0].expression,
    };
}
function skipBlank(docs) {
    for (let i = docs.length - 1; i >= 0; i--) {
        const doc = docs[i];
        if (typeof doc !== 'string') {
            if (doc.type === 'break-parent') {
                continue;
            }
        }
        return i;
    }
    return -1;
}
function nukeLastLine(doc) {
    if (typeof doc === 'string') {
        return doc;
    }
    switch (doc.type) {
        case 'concat':
            const end = skipBlank(doc.parts);
            if (end > -1) {
                return concat$1([
                    ...doc.parts.slice(0, end),
                    nukeLastLine(doc.parts[end]),
                    ...doc.parts.slice(end + 1),
                ]);
            }
            break;
        case 'line':
            return '';
    }
    return doc;
}

function locStart(node) {
    return node.start;
}
function locEnd(node) {
    return node.end;
}
const languages = [
    {
        name: 'svelte',
        parsers: ['svelte'],
        extensions: ['.svelte', '.html'],
    },
];
const parsers = {
    svelte: {
        parse: (text) => svelte.parse(text),
        locStart,
        locEnd,
        astFormat: 'svelte-ast',
    },
};
const printers = {
    'svelte-ast': {
        print,
        embed,
    },
};

exports.languages = languages;
exports.parsers = parsers;
exports.printers = printers;
//# sourceMappingURL=plugin.js.map
